def make_url_part(name, value, value_type):
    if value == None:
        return ''
    if type(value) != value_type:
        raise ValueError('{}={} must be {}'.format(
            name, value, value_type))
    result = "&${}={}".format(name, value)
    return result



import json

import requests

from odata1cw.core import Infobase
from odata1cw.postingmode import PostingMode
from odata1cw.utils import make_url_part


class Document:
    infobase: Infobase
    docname: str
    url: str

    def __init__(self, infobase, docname):
        self.infobase = infobase
        self.docname = docname
        self.url = self.infobase._full_url.format(obj='Document_'+self.docname)

    def query(self, top=None, skip=None, select=None, odata_filter=None, expand=None):
        _url_top = make_url_part('top', top, int)
        _url_skip = make_url_part('skip', skip, int)
        _url_select = make_url_part('select', select, str)
        _url_filter = make_url_part('filter', odata_filter, str)
        _url_expand = make_url_part('expand', expand, str)

        url = self.url + _url_top + _url_skip + _url_select + _url_filter + _url_expand

        r = requests.get(url, auth=self.infobase._auth,
                         headers=self.infobase._headers)
        if(r.status_code != 200):
            raise Exception(r.text)
        return json.loads(r.text)['value']

    # TODO documents $count
    # TODO documents $inlinecount
    # TODO documents $orderby
    # TODO documents $expand

    def get(self, guid, select=None):
        obj = "Document_{}(guid'{}')".format(self.docname, guid)
        url = self.infobase._full_url.format(obj=obj)
        _url_select = make_url_part('select', select, str)
        url = url + _url_select
        r = requests.get(url, auth=self.infobase._auth,
                         headers=self.infobase._headers)
        if(r.status_code != 200):
            raise Exception(r.text)
        return json.loads(r.text)

    def post(self, guid, posting_mode):
        obj = "Document_{}(guid'{}')/Post".format(self.docname, guid)
        url = self.infobase._full_url.format(obj=obj)
        if posting_mode == PostingMode.OPER:
            url = url + '&PostingModeOperational=true'
        elif posting_mode == PostingMode.POST:
            url = url + '&PostingModeOperational=false'
        else:
            raise ValueError('Use unpost_document() for unposting')
        r = requests.post(url, auth=self.infobase._auth,
                          headers=self.infobase._headers)
        if(r.status_code != 200):
            raise Exception(r.text)

    def unpost(self, guid):
        obj = "Document_{}(guid'{}')/Unpost".format(self.docname, guid)
        url = self.infobase._full_url.format(obj=obj)
        r = requests.post(url, auth=self.infobase._auth,
                          headers=self.infobase._headers)
        if(r.status_code != 200):
            raise Exception(r.text)

    def edit(self, guid, data):
        # Because Posting = True does not create records in registers
        if 'Posting' in data:
            raise ValueError('Do not pass the "Posting" field')
        obj = "Document_{}(guid'{}')".format(self.docname, guid)
        url = self.infobase._full_url.format(obj=obj)
        r = requests.patch(url, auth=self.infobase._auth,
                           headers=self.infobase._headers, data=json.dumps(data))
        if(r.status_code != 200):
            raise Exception(r.text)

        # TODO Check whether document changes register records (rePosted)?
        return json.loads(r.text)

    def create(self, data, posting_mode=PostingMode.UNPOST):
        # Because Posting = True does not create records in registers
        if ('Posting' in data) and (data['Posting'] == True):
            raise ValueError('Do not pass the "Posting" field')
        if not 'Date' in data:
            raise ValueError('Date value cannot be an empty date')

        r = requests.post(self.url, auth=self.infobase._auth,
                          headers=self.infobase._headers, data=json.dumps(data))
        if r.status_code != 201:
            raise Exception(r.text)

        new_doc = json.loads(r.text)

        if posting_mode != PostingMode.UNPOST:
            self.post(new_doc['Ref_Key'], posting_mode)

        return new_doc



import json

import requests

from odata1cw.core import Infobase
from odata1cw.utils import make_url_part


class Catalog:
    infobase: Infobase

    def __init__(self, infobase, catname):
        self.infobase = infobase
        self.catname = catname
        self.url = self.infobase._full_url.format(obj='Catalog_'+self.catname)

    def get(self, guid, select=None):
        obj = "Catalog_{}(guid'{}')".format(self.catname, guid)
        url = self.infobase._full_url.format(obj=obj)
        _url_select = make_url_part('select', select, str)
        url = url + _url_select
        r = requests.get(url, auth=self.infobase._auth,
                         headers=self.infobase._headers)
        if(r.status_code != 200):
            raise Exception(r.text)
        return json.loads(r.text)

    def query(self, top=None, skip=None, select=None, odata_filter=None, expand=None):
        _url_top = make_url_part('top', top, int)
        _url_skip = make_url_part('skip', skip, int)
        _url_select = make_url_part('select', select, str)
        _url_filter = make_url_part('filter', odata_filter, str)
        _url_expand = make_url_part('expand', expand, str)
        url = self.url + _url_top + _url_skip + _url_select + _url_filter + _url_expand
        r = requests.get(url, auth=self.infobase._auth,
                         headers=self.infobase._headers)
        if(r.status_code != 200):
            raise Exception(r.text)
        return json.loads(r.text)['value']

    def create(self, data):
        r = requests.post(self.url, auth=self.infobase._auth,
                          headers=self.infobase._headers, data=json.dumps(data))
        if r.status_code != 201:
            raise Exception(r.text)
        return json.loads(r.text)

    def edit(self, guid, data):
        obj = "Catalog_{}(guid'{}')".format(self.catname, guid)
        url = self.infobase._full_url.format(obj=obj)
        r = requests.patch(url, auth=self.infobase._auth,
                           headers=self.infobase._headers, data=json.dumps(data))
        if(r.status_code != 200):
            raise Exception(r.text)

        return json.loads(r.text)



class PostingMode:
    """This object represents a 1C document posting modes ."""

    UNPOST = 'UNPOST'
    """:obj:`str`: 'Markdown'"""
    POST = 'POST'
    """:obj:`str`: 'HTML'"""
    OPER = 'OPER'
    """:obj:`str`: 'HTML'"""



import requests



class Infobase:
    _headers = {
        'Content-Type': 'application/json',
        'Accept': 'application/json',
        'Connection': 'keep-alive'
    }

    server: str
    infobase: str
    _full_url: str
    _auth: str

    def __init__(self, server, infobase, username, password):
        self.server = server
        self.infobase = infobase
        self._full_url = server + infobase + \
            '/odata/standard.odata/{obj}?$format=json'
        self._auth = requests.auth.HTTPBasicAuth(username, password)



import json

import requests

from odata1cw.core import Infobase
from odata1cw.utils import make_url_part


class InformationRegister:
    infobase: Infobase

    def __init__(self, infobase, regname):
        self.infobase = infobase
        self.regname = regname
        self.url = self.infobase._full_url.format(
            obj='InformationRegister_'+self.regname)

    def query(self, top=None, skip=None, select=None, odata_filter=None, expand=None):
        _url_top = make_url_part('top', top, int)
        _url_skip = make_url_part('skip', skip, int)
        _url_select = make_url_part('select', select, str)
        _url_filter = make_url_part('filter', odata_filter, str)
        _url_expand = make_url_part('expand', expand, str)
        url = self.url + _url_top + _url_skip + _url_select + _url_filter + _url_expand
        r = requests.get(url, auth=self.infobase._auth,
                         headers=self.infobase._headers)
        if(r.status_code != 200):
            raise Exception(r.text)
        return json.loads(r.text)['value']

    def slice_last(self, period=None, condition=None, select=None, orderby=None, expand=None):
        _url_select = make_url_part('select', select, str)
        _url_orderby = make_url_part('orderby', orderby, str)
        _url_expand = make_url_part('expand', expand, str)
        period_value = '' if period is None else period
        condition_value = '' if condition is None else condition

        full_url = (self.infobase._full_url.format(
            obj='InformationRegister_'+self.regname+f'/SliceLast({period_value},{condition_value})'))+f'{_url_select}{_url_orderby}{_url_expand}'

        r = requests.get(full_url, auth=self.infobase._auth,
                         headers=self.infobase._headers)
        if(r.status_code != 200):
            raise Exception(r.text)
        return json.loads(r.text)['value']



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="odata1cw",
    version="0.0.4",
    author="Ilia Belov",
    author_email="belov.penrose@gmail.com",
    description="1C-Odata wrapper",
    long_description="1C (v8.1c.ru) OData wrapper.",
    long_description_content_type="text/markdown",
    url="https://github.com/belov38/1c-odata",
    packages=setuptools.find_packages(),
    install_requires=[
          'requests',
      ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)